/**
	$Id: StylableTag.java 5900 2011-12-07 13:43:15Z nicklas $

	Copyright (C) 2011 Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
*/
package net.sf.basedb.clients.web.taglib;

import javax.servlet.jsp.tagext.TagSupport;


/**
	Generic tag class for tags that want to support 
	id, class and style of the main html object they 
	use. The class can be specified as a main class and
	a subclass which is added instead of replacing the
	main class.

	@author Nicklas
	@since 3.1
	@base.modified $Date: 2011-12-07 14:43:15 +0100 (Wed, 07 Dec 2011) $
*/
public abstract class StylableTag
	extends TagSupport
{

	private static final long serialVersionUID = -383421425096524117L;
	private String clazz;
	private String subclass;
	private String style;
	
	/**
		Create a new stylable tag.
		@param defaultClass The default class to use if the tag doesn't
			override with it's own value
	 */
	protected StylableTag(String defaultClass)
	{
		this.clazz = defaultClass;
	}
	
	public void setClazz(String clazz)
	{
		this.clazz = clazz;
	}
	public String getClazz()
	{
		return clazz;
	}
	
	public void setSubclass(String subclass)
	{
		this.subclass = subclass;
	}
	public String getSubclass()
	{
		return subclass;
	}
	
	public void setStyle(String style)
	{
		this.style = style;
	}
	public String getStyle()
	{
		return style;
	}

	/**
		Get the full class (eg. main and subclass) as a single
		string.
		@param extraClasses Additional classes that should also be added
		@return The combined class or null if no class has been set
	*/
	public String getFullClass(String...extraClasses)
	{
		StringBuilder fullClass = new StringBuilder();
		if (getClazz() != null)
		{
			fullClass.append(getClazz());
		}
		if (getSubclass() != null)
		{
			if (fullClass.length() > 0) fullClass.append(" ");
			fullClass.append(getSubclass());
		}
		if (extraClasses != null && extraClasses.length > 0)
		{
			for (String extra: extraClasses)
			{
				if (extra != null)
				{
					if (fullClass.length() > 0) fullClass.append(" ");
					fullClass.append(extra);
				}
			}
		}
		return fullClass.length() == 0 ? null : fullClass.toString();
	}
	
	/**
		Add id, class and style attributes the the stringbuilder.
	*/
	protected void addIdAndStyles(StringBuilder sb, String... extraClasses)
	{
		if (getId() != null) sb.append(" id=\"").append(getId()).append("\"");
		String fullClass = getFullClass(extraClasses);
		if (fullClass != null) sb.append(" class=\"").append(fullClass).append("\"");
		if (getStyle() != null) sb.append(" style=\"").append(getStyle()).append("\"");
	}
	
}
