/*
	$Id: HeaderRow.java 5948 2012-02-08 13:16:27Z nicklas $

	Copyright (C) 2012 Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
*/
package net.sf.basedb.clients.web.taglib.table;

import javax.servlet.jsp.JspException;
import javax.servlet.jsp.JspTagException;

import net.sf.basedb.clients.web.taglib.StylableTag;

/**
	Container tag for a single header row in a table.

	<p>
	<b>Syntax:</b>
	<pre>
   &lt;tbl:headerrow&gt;
      &lt;tbl:header ... &gt;
   &lt;/tbl:headerrow&gt;
</pre>

	@author Nicklas
	@since 3.1
*/
public class HeaderRow
	extends StylableTag
{

	private static final long serialVersionUID = -6286111811069582940L;

	public HeaderRow()
	{
		super("headerrow");
	}
	
	private int numColumns;
	
	void addHeaderColumns(int numColumns)
	{
		this.numColumns += numColumns;
	}
	
	/*
		Display methods
	*/
	public int doStartTag()
		throws JspException
	{
		numColumns = 0;
		StringBuilder sb = new StringBuilder();
		sb.append("<tr");
		addIdAndStyles(sb);
		sb.append(">\n");
		try
		{
			pageContext.getOut().print(sb.toString());
		}
		catch (Exception ex)
		{
			throw new JspTagException(ex.getMessage());
		}
		return EVAL_BODY_INCLUDE;
	}

	public int doEndTag()
		throws JspException
	{
		Table table = (Table)findAncestorWithClass(this, Table.class);
		if (table != null)
		{
			table.setNumColumns(numColumns);
		}

		StringBuilder sb = new StringBuilder();
		sb.append("</tr>\n");
		try
		{
			pageContext.getOut().print(sb.toString());
		}
		catch (Exception ex)
		{
			throw new JspTagException(ex.getMessage());
		}
		return EVAL_PAGE;
	}
}

