/*
	$Id: ItemParameterType.java 5437 2010-10-01 11:51:57Z nicklas $

	Copyright (C) 2005 Samuel Andersson, Johan Enell, Nicklas Nordborg
	Copyright (C) 2006 Jari Häkkinen, Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
*/
package net.sf.basedb.core;

import net.sf.basedb.core.data.ItemParameterValueData;
import java.util.List;

/**
	This class represent a parameter type that is an object derived from 
	{@link net.sf.basedb.core.data.BasicData}.
	It will have a list of <code>BasicItem</code> derived objects of the same type
	to choose from.

	@author Samuel, Nicklas
	@version 2.0
	@base.modified $Date: 2010-10-01 13:51:57 +0200 (Fri, 01 Oct 2010) $
*/
public class ItemParameterType<T extends BasicItem>
	extends ParameterType<T>
{
	
	/**
		Create a new item parameter type, allowing null values.
		@param clazz The class object of the type of items that are accepted
	 	@param items List of items to choose from. 
	*/
	public ItemParameterType(Class<T> clazz, List<T> items)
	{
		this(clazz, null, false, 1, items);
	}
	

	public ItemParameterType(Class<T> clazz, T defaultValue, boolean notNull, int multiplicity, List<T> items)
	{
		super(null, clazz, defaultValue, notNull, multiplicity, 0, 0, items);
	}

	/*
		From the Object class
		-------------------------------------------
	*/
	public String toString()
	{
		return "ItemParameterType: Items=" + getItems();
	}
	// -------------------------------------------

	/*
		From the ParameterType class
		-------------------------------------------
	*/
	/**
		Check if the value is in the list of allowed values.
	*/
	@Override
	void validateValue(String name, T value)
		throws InvalidDataException
	{
	}

	/**
		Create a new {@link ItemParameterValueData} object.
	*/
	@Override
	ItemParameterValueData newParameterValueData()
	{
		return new ItemParameterValueData();
	}
	// -------------------------------------------

}
