/**
	$Id: SpecialQuery.java 5662 2011-06-20 11:55:13Z nicklas $

	Copyright (C) 2011 Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
*/
package net.sf.basedb.core;

import java.util.ArrayList;
import java.util.List;

import net.sf.basedb.core.query.Hql;
import net.sf.basedb.core.query.ResultList;
import net.sf.basedb.core.query.Selects;
import net.sf.basedb.util.transform.Transformer;

/**
	A special HQL query implementation that can return almost any kind of object.

	@author Nicklas
	@since 3.0
	@base.modified $Date: 2011-06-20 13:55:13 +0200 (Mon, 20 Jun 2011) $
*/
public class SpecialQuery<C>
	extends AbstractHqlQuery
{

	private final Transformer<C, Object> transformer;
	
	@SuppressWarnings("unchecked")
	SpecialQuery(String select, String from, String alias, Transformer<C, ?> transformer)
	{
		super(from, alias, false);
		super.selectPermanent(Selects.expression(Hql.property(alias, select), null, true));
		this.transformer = (Transformer<C, Object>)transformer;
	}
	
	/**
		Get the result of the query as a list.
		@param dc The DbControl to use for database access
	*/
	@SuppressWarnings("unchecked")
	public ResultList<C> list(DbControl dc)
	{
		SessionControl sc = dc.getSessionControl();
		List<C> transformedResults = null;
		List results = HibernateUtil.loadList(Object.class, getMainHqlQuery(dc), sc);
		long totalCount = results.size();
		
		// Load the total count if needed and requested
		if ((getFirstResult() >= 0 || getMaxResults() > 0) && isReturningTotalCount())
		{
			totalCount = HibernateUtil.loadData(Long.class, getCountHqlQuery(dc));
		}

		// Transform the results if a transformer has been given
		if (transformer != null)
		{
			transformedResults = new ArrayList<C>(results.size());
			for (Object x : results)
			{
				transformedResults.add(transformer.transform(dc, x));
			}
		}
		else
		{
			transformedResults = (List<C>)results;
		}
		
		return new UnmodifiableResultList<C>(transformedResults, totalCount);
	}
	
}
