/**
	$Id: BioPlateEventParticipantData.java 5818 2011-10-21 11:15:41Z nicklas $

	Copyright (C) 2010 Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
*/
package net.sf.basedb.core.data;

import java.util.HashSet;
import java.util.Set;

/**
	Registers a plate as a participant in a bioplate event. Each participant may
	have a 'role' that is simply a free textual description of how the plate was used.
	Eg. for a 'move' event there can be 'source' and 'destination' plates. If the
	ordering of plates are important it is also possible to specify an index value.
	The index doesn't have to unique, but a plate can only be a registered as a 
	participant once per event.
	
	@author Nicklas
	@since 2.17
 	@see net.sf.basedb.core.BioPlateEventParticipant
	@see <a href="../../../../../../html/developer/api/data_api.biomaterials.html">Developer documentation: Biomaterials LIMS</a>
	@base.modified $Date: 2011-10-21 13:15:41 +0200 (Fri, 21 Oct 2011) $
	@hibernate.class table="`BioPlateEventParticipants`" lazy="true"
*/
public class BioPlateEventParticipantData
	extends BasicData
{
	
	public BioPlateEventParticipantData()
	{}

	
	private BioPlateData bioPlate;
	/**
		The bioplate that is participating in the event.
		@hibernate.many-to-one column="`bioplate_id`" not-null="true" outer-join="false" update="false" unique-key="uniqueevent"
	*/
	public BioPlateData getBioPlate()
	{
		return bioPlate;
	}
	public void setBioPlate(BioPlateData bioPlate)
	{
		this.bioPlate = bioPlate;
	}

	private BioPlateEventData event;
	/**
		The bioplate event.
		@hibernate.many-to-one column="`event_id`" not-null="true" outer-join="false" update="false" unique-key="uniqueevent"
	*/
	public BioPlateEventData getEvent()
	{
		return event;
	}
	public void setEvent(BioPlateEventData event)
	{
		this.event = event;
	}
	
	private int index;
	/**
		Get the index of this plate in the event. The index may or may not
		be unique depending on the event.
		@hibernate.property column="`index`" type="int" not-null="true"
	*/
	public int getIndex()
	{
		return index;
	}

	public void setIndex(int index)
	{
		this.index = index;
	}

	public static final int MAX_ROLE_LENGTH = 255;
	private String role;
	/**
		Get the role of the plate in the event. The role is only a textual description
		and is not of concern to the core. The role can be null.
		@hibernate.property column="`role`" type="string" length="255" not-null="false"
	*/
	public String getRole()
	{
		return role;
	}
	public void setRole(String role)
	{
		this.role = role;
	}

	private Set<BioMaterialEventData> bioMaterialEvents;
	/**
		This is the inverse end.
		@see BioMaterialEventData#getBioPlateEventParticipant()
		@hibernate.set lazy="true" inverse="true"
		@hibernate.collection-key column="`bioplateeventparticipant_id`"
		@hibernate.collection-one-to-many class="net.sf.basedb.core.data.BioMaterialEventData"
	*/
	public Set<BioMaterialEventData> getBioMaterialEvents()
	{
		if (bioMaterialEvents == null) 
		{
			bioMaterialEvents = new HashSet<BioMaterialEventData>();
		}
		return bioMaterialEvents;
	}
	void setBioMaterialEvents(Set<BioMaterialEventData> bioMaterialEvents)
	{
		this.bioMaterialEvents = bioMaterialEvents;
	}
}
