/**
	$Id: DerivedBioAssayData.java 5818 2011-10-21 11:15:41Z nicklas $

	Copyright (C) 2011 Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
*/
package net.sf.basedb.core.data;

import java.util.Date;
import java.util.Set;

/**

	@author Nicklas
	@since 3.0
	@base.modified $Date: 2011-10-21 13:15:41 +0200 (Fri, 21 Oct 2011) $
	@hibernate.class table="`DerivedBioAssays`" lazy="true"
	@see <a href="../../../../../../html/developer/api/data_api.bioassays.html">Developer documentation: Bioassays and raw data</a>
*/
public class DerivedBioAssayData
	extends AnnotatedData
	implements RegisteredData, FileStoreEnabledData, SubtypableData
{

	public DerivedBioAssayData()
	{}
	
	/*
		From the SubtypableData interface
		----------------------------------
	*/
	private ItemSubtypeData subtype;
	@Override
	public ItemSubtypeData getItemSubtype()
	{
	   return subtype;
	}
	@Override
	public void setItemSubtype(ItemSubtypeData subtype)
	{
	   this.subtype = subtype;
	}
	// ------------------------------

	/*
		From the FileStoreEnabledData interface
		-------------------------------------------
	*/
	private FileSetData fileSet;
	@Override
	public FileSetData getFileSet()
	{
	   return fileSet;
	}
	@Override
	public void setFileSet(FileSetData fileSet)
	{
	   this.fileSet = fileSet;
	}
	// -------------------------------------------
	/*
		From the RegisteredData interface
		-------------------------------------------
	*/
	private Date entryDate;
	@Override
	public Date getEntryDate()
	{
		return entryDate;
	}
	public void setEntryDate(Date entryDate)
	{
		this.entryDate = entryDate;
	}
	// -------------------------------------------

	private PhysicalBioAssayData bioAssay;
	/**
		Get the physical bioassay that is the direct or indirect parent bioassay set.
		@hibernate.many-to-one column="`bioassay_id`" not-null="true" update="false" outer-join="false"
	*/
	public PhysicalBioAssayData getPhysicalBioAssay()
	{
		return bioAssay;
	}
	public void setPhysicalBioAssay(PhysicalBioAssayData bioAssay)
	{
		this.bioAssay = bioAssay;
	}

	private DerivedBioAssayData parent;
	/**
		Get the bioassay that is the parent of this bioassay. Null if this bioassay was
		created from a physical bioassay.
		@hibernate.many-to-one column="`parent_id`" not-null="false" update="false" outer-join="false"
	*/
	public DerivedBioAssayData getParent()
	{
		return parent;
	}
	public void setParent(DerivedBioAssayData parent)
	{
		this.parent = parent;
	}

	private Set<DerivedBioAssayData> children;
	/**
		This is the inverse end.
		@see DerivedBioAssayData#getParent()
		@hibernate.set lazy="true" inverse="true" 
		@hibernate.collection-key column="`parent_id`"
		@hibernate.collection-one-to-many class="net.sf.basedb.core.data.DerivedBioAssayData"
	*/
	Set<DerivedBioAssayData> getChildren()
	{
		return children;
	}
	void setChildren(Set<DerivedBioAssayData> children)
	{
		this.children = children;
	}

	private Set<RawBioAssayData> rawBioAssays;
	/**
		This is the inverse end.
		@see RawBioAssayData#getParentBioAssay()
		@hibernate.set lazy="true" inverse="true" 
		@hibernate.collection-key column="`bioassay_id`"
		@hibernate.collection-one-to-many class="net.sf.basedb.core.data.RawBioAssayData"
	*/
	Set<RawBioAssayData> getRawBioAssays()
	{
		return rawBioAssays;
	}
	void setRawBioAssays(Set<RawBioAssayData> rawBioAssays)
	{
		this.rawBioAssays = rawBioAssays;
	}

	
	private ProtocolData protocol;
	/**
		Get the protocol used to create this bioassay set.
		@hibernate.many-to-one column="`protocol_id`" not-null="false" outer-join="false"
	*/
	public ProtocolData getProtocol()
	{
		return protocol;
	}
	public void setProtocol(ProtocolData protocol)
	{
		this.protocol = protocol;
	}

	private HardwareData hardware;
	/**
	 	Get the hardware used to create this bioassay set.
	 	@hibernate.many-to-one column="`hardware_id`" not-null="false" outer-join="false"
	*/
	public HardwareData getHardware()
	{
		return hardware;
	}
	public void setHardware(HardwareData hardware)
	{
		this.hardware = hardware;
	}

	private SoftwareData software;
	/**
	 	Get the software used to create this bioassay set.
	 	@hibernate.many-to-one column="`software_id`" not-null="false" outer-join="false"
	*/
	public SoftwareData getSoftware()
	{
		return software;
	}
	public void setSoftware(SoftwareData software)
	{
		this.software = software;
	}

	private JobData job;
	/**
	 	Get the job used to create this bioassay set.
	 	@hibernate.many-to-one column="`job_id`" not-null="false" update="false" outer-join="false"
	*/
	public JobData getJob()
	{
		return job;
	}
	public void setJob(JobData job)
	{
		this.job = job;
	}

	private ExtractData extract;
	/**
	 	Get the extract that this bioassay is related to.
	 	@hibernate.many-to-one column="`extract_id`" not-null="false" outer-join="false"
	*/
	public ExtractData getExtract()
	{
		return extract;
	}
	public void setExtract(ExtractData extract)
	{
		this.extract = extract;
	}
	
}
