/*
  $Id: PropertyFilterData.java 5818 2011-10-21 11:15:41Z nicklas $

	Copyright (C) 2005 Nicklas Nordborg
	Copyright (C) 2006 Jari Häkkinen, Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

  BASE is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 3
  of the License, or (at your option) any later version.

  BASE is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with BASE. If not, see <http://www.gnu.org/licenses/>.
*/
package net.sf.basedb.core.data;

/**
	This class holds context information to help client applications
	with storing common settings for tables.
	
	@author Nicklas
	@version 2.0
	@see net.sf.basedb.core.ItemContext
	@see <a href="../../../../../../html/developer/api/data_api.clients.html">Developer documentation: Client, session and settings</a>
	@base.modified $Date: 2011-10-21 13:15:41 +0200 (Fri, 21 Oct 2011) $
*/
public class PropertyFilterData
{
	public PropertyFilterData()
	{}

	private int operator;
	/**
		Get the operator used for this filter.
		@hibernate.property column="`operator`" type="int" not-null="true"
	*/
	public int getOperator()
	{
		return operator;
	}
	public void setOperator(int operator)
	{
		this.operator = operator;
	}

	private int valueType;
	/**
		Get the type of value user for this filter.
		@hibernate.property column="`value_type`" type="int" not-null="true"
	*/
	public int getValueType()
	{
		return valueType;
	}
	public void setValueType(int valueType)
	{
		this.valueType = valueType;
	}

	/**
		The maximum length of the value that can be stored in the database.
		@see #setValue(String)
	*/
	public static final int MAX_VALUE_LENGTH = 65535;
	private String value;
	/**
		Get the value used in this filter.
		@hibernate.property column="`value`" type="text" not-null="false"
	*/
	public String getValue()
	{
		return value;
	}
	public void setValue(String value)
	{
		this.value = value;
	}
	
	private UnitData unit;
	/**
		@hibernate.many-to-one column="`unit_id`" not-null="false"
		@since 2.9
	*/
	public UnitData getUnit()
	{
		return unit;
	}
	public void setUnit(UnitData unit)
	{
		this.unit = unit;
	}

}
