/**
	$Id: DefaultDbEngine.java 5889 2011-11-25 12:07:04Z nicklas $

	Copyright (C) 2006, 2007 Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or modify it
	under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 3 of the License, or
	(at your option) any later version.

	BASE is distributed in the hope that it will be useful, but
	WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
	General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
*/
package net.sf.basedb.core.dbengine;

import java.util.Set;

import org.hibernate.mapping.PersistentClass;

/**
 	A default DbEngine that is selected if no other can be found. This
 	probably has some issues with most databases.

	@author nicklas
	@version 2.0
	@base.modified $Date: 2011-11-25 13:07:04 +0100 (Fri, 25 Nov 2011) $
*/
public class DefaultDbEngine
	extends AbstractDbEngine
{

	/**
		Create DefaultDbEngine.
	*/
	public DefaultDbEngine()
	{}

	/*
		From the DbEngine interface
		-------------------------------------------
	*/
	/**
		Generates 
		<code>CREATE [UNIQUE] INDEX &lt;name&gt; ON 
		&lt;catalog&gt;.&lt;schema&gt;.&lt;table&gt; (&lt;columns&gt;)</code>
		which should work for most databases.
	*/
	public String getCreateIndexSql(String catalog, String schema, String table, String name,
			Set<String> columns, boolean unique)
	{
		StringBuilder sql = new StringBuilder();
		sql.append("CREATE ");
		if (unique) sql.append("UNIQUE ");
		sql.append("INDEX ").append(getQuotedName(name));
		sql.append(" ON ");
		if (catalog != null) sql.append(getQuotedName(catalog)).append(".");
		if (schema != null) sql.append(getQuotedName(schema)).append(".");
		sql.append(getQuotedName(table)).append(" (");
		int i = 0;
		for (String column : columns)
		{
			if (i > 0) sql.append(", ");
			sql.append(getQuotedName(column));
			i++;
		}
		sql.append(")");
		return sql.toString();
	}

	/**
	 	Generate <code>DROP INDEX &lt;catalog&gt;.&lt;schema&gt;.&lt;table&gt;.&lt;name&gt;</code>.
	*/
	public String getDropIndexSql(String catalog, String schema, String table, String name, boolean unique)
	{
		StringBuilder sql = new StringBuilder();
		sql.append("DROP INDEX ");
		if (catalog != null) sql.append(getQuotedName(catalog)).append(".");
		if (schema != null) sql.append(getQuotedName(schema)).append(".");
		sql.append(getQuotedName(table)).append(".").append(getQuotedName(name));
		return sql.toString();
	}
	
	/**
		Generate <code>ALTER TABLE &lt;catalog&gt;.&lt;schema&gt;.&lt;table&gt; DROP CONSTRAINT &lt;name&gt;</code>
		@since 3.0
	*/
	@Override
	public String getDropForeignKeySql(String catalog, String schema, String table, String name)
	{
		StringBuilder sql = new StringBuilder();
		sql.append("ALTER TABLE ");
		if (catalog != null) sql.append(getQuotedName(catalog)).append(".");
		if (schema != null) sql.append(getQuotedName(schema)).append(".");
		sql.append(getQuotedName(table));
		sql.append(" DROP CONSTRIANT ");
		sql.append(getQuotedName(name));
		return sql.toString();
	}

	
	/**
		@return Always "null" (assuming that the database knows how to generate the id)
	*/
	@Override
	public String getInsertAutoIncrementSql(PersistentClass pc)
	{
		return "null";
	}

	/**
		Return false.
	*/
	public boolean analyzeAfterBatchOperation()
	{
		return false;
	}
	
	/**
		Return null.
	*/
	public String getOptimizeTableSql(String catalog, String schema, String table)
	{
		return null;
	}
	
	/**
		Return null.
	*/
	public String getAnalyzeTableSql(String catalog, String schema, String table)
	{
		return null;
	}
	
	/**
		Returns TRUE.
		@since 2.4
	*/
	public boolean caseInsensitiveComparison()
	{
		return true;
	}
	// -------------------------------------------

	@Override
	public String getCreateForeignKeySql(String catalog, String schema,
			String table, String name, Set<String> columns, String refTable,
			Set<String> refColumns) 
	{
		throw new UnsupportedOperationException();
	}

	@Override
	public String getCreatePrimaryKeySql(String catalog, String schema,
			String table, String name, Set<String> columns) 
	{
		throw new UnsupportedOperationException();
	}

	@Override
	public String getDropPrimaryKey(String catalog, String schema,
			String table, String name)
	{
		throw new UnsupportedOperationException();
	}


}
