/*
	$Id: HqlQuery.java 5564 2011-02-04 08:25:16Z nicklas $

	Copyright (C) 2005 Nicklas Nordborg
	Copyright (C) 2006 Jari Häkkinen
	Copyright (C) 2007 Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
*/
package net.sf.basedb.core.query;

import net.sf.basedb.core.BasicItem;
import net.sf.basedb.core.InvalidDataException;

/**
	This is a query that is using HQL (Hibernate Query Language) as
	the query language. All implementors of this interface must return 
	{@link QueryType#HQL} from the {@link Query#getQueryType()} method.
	
	@author Nicklas
	@version 2.0
	@base.modified $Date: 2011-02-04 09:25:16 +0100 (Fri, 04 Feb 2011) $
*/
public interface HqlQuery
	extends Query
{

	/**
		Specify if the query results should be cached or not. 
		
		@base.developer 
		This option requires that a secondary cache that supports query results
		are used (ie. EHCache).
		@param flag TRUE if the query results should be cached, FALSE otherwise
		@throws UnsupportedOperationException If this operation isn't supported by
			the implementation
	*/
	public void setCacheResult(boolean flag);
	
	/**
		If the query results are cached or not.
		@see #setCacheResult(boolean)
	*/
	public boolean isCachingResult();

	/**
		Set the value of an entity parameter. If the value of this parameter
		has already been set permanently, an <code>InvalidDataException</code> is thrown.
		@param name The name of the parameter
		@param value The value of the parameter
		@throws UnsupportedOperationException If this operation isn't supported by
			the implementation
		@throws InvalidDataException If the value of the parameter has already 
			been permanently set
		@since 2.17
	*/
	public void setEntityParameter(String name, BasicItem value);

	/**
		Permanently set the value of an entity parameter. If the value of this parameter
		has already been set permanently, an <code>InvalidDataException</code> is thrown.
		@param name The name of the parameter
		@param value The value of the parameter
		@throws UnsupportedOperationException If this operation isn't supported by
			the implementation
		@throws InvalidDataException If the value of the parameter has already 
			been permanently set
		@since 2.17
	*/
	public void setPermanentEntityParameter(String name, BasicItem value);

}
