/**
	$Id $

	Copyright (C) 2010 Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or modify it
	under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 3 of the License, or
	(at your option) any later version.

	BASE is distributed in the hope that it will be useful, but
	WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
	General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
*/
package net.sf.basedb.core.query;

import java.util.Collection;
import java.util.Collections;

import net.sf.basedb.core.BaseException;
import net.sf.basedb.core.DbControl;
import net.sf.basedb.core.HibernateUtil;
import net.sf.basedb.util.EqualsHelper;

/**
	Get the date part of a date or timestamp epxression.

	@author Nicklas
	@since 2.16
	@see Expressions#toDate(Expression)
	@base.modified $Date $
*/
public class ToDateExpression
	implements Expression
{
	private final Expression e1;

	ToDateExpression(Expression e1)
	{
		assert e1 != null : "e1 == null";
		this.e1 = e1;
	}

	/*
		From the QueryElement interface
		-------------------------------------------
	*/
	public String toQl(Query query, DbControl dc)
		throws BaseException
	{
		return HibernateUtil.getDbEngine().castToDate(e1.toQl(query, dc));
	}
	/**
		@return The expression to use for the calculation
	*/
	public Collection<? extends QueryElement> getChildren()
	{
		return Collections.singleton(e1);
	}
	// -------------------------------------------
	/*
		From the Object class
		-------------------------------------------
	*/
	@Override
	public String toString()
	{
		return "cast (" + e1 + " as date)";
	}
	/**
		This expression is equal to another ToDateExpression if their 
		child expressions are equal.
	*/
	@Override
	public boolean equals(Object other)
	{
		if (this == other) return true;
		if (other == null || this.getClass() != other.getClass()) return false;
		ToDateExpression o = (ToDateExpression)other;
		return EqualsHelper.equals(e1, o.e1);
	}
	@Override
	public int hashCode()
	{
		return EqualsHelper.hashCode(e1) + getClass().hashCode();
	}
	// -------------------------------------------
}