/**
	$Id: EventType.java 5193 2009-11-27 13:06:17Z nicklas $

	Copyright (C) 2009 Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
*/
package net.sf.basedb.util.bfs;

/**
	Defines an event type that is sent in event notifications to event
	handlers. The parser that defines the event type is recommended to
	keep a <code>public static final EventType ...</code> constant for
	each event type it defines. Event handlers can then simply reference
	this constant to determine which event that was fired. Eg.
	<pre>
public void handleEvent(EventType eventType, Object eventData)
{
	if (eventType == Parser.PRE_DEFINED_TYPE_1)
	{
	   // Do something
	}
	else if (eventType == Parser.PRE_DEFINED_TYPE_2)
	{
		// Do something else
	}
}
</pre>
	
	@author Nicklas
	@version 2.15
	@base.modified $Date: 2009-11-27 14:06:17 +0100 (Fri, 27 Nov 2009) $
	@see EventHandler
*/
public class EventType<T>
{
	
	private final String id;

	/**
		Create a new event type object.
	*/
	public EventType(String id)
	{
		this.id = id;
	}
	
	@Override
	public String toString()
	{
		return "EventType(" + id + ")";
	}
	
}
