/**
	$Id: ValidAndNewOrModifiedFilter.java 5607 2011-04-15 07:52:28Z nicklas $

	Copyright (C) 2011 Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
*/
package net.sf.basedb.util.extensions.manager.filter;

import java.io.File;

import net.sf.basedb.util.extensions.manager.ExtensionsFile;
import net.sf.basedb.util.extensions.manager.Settings;
import net.sf.basedb.util.filter.Filter;

/**
	Filter implementation for extension files that 
	allow installed files to pass. A {@link Settings}
	object is used to keep track of which files are
	installed and not.
	
	@author Nicklas
	@since 3.0
	@base.modified $Date: 2011-04-15 09:52:28 +0200 (fr, 15 apr 2011) $
*/
public class InstalledFilter
	implements Filter<ExtensionsFile>
{
	
	private static final org.apache.log4j.Logger log = 
		org.apache.log4j.LogManager.getLogger("net.sf.basedb.util.extensions.manager.filter.InstalledFilter");

	private final Settings settings;
	private final Filter<ExtensionsFile> parent;
	
	/**
		Create a new filter.
		@param settings The settings object to use when checking if a file is installed or not
	*/
	public InstalledFilter(Settings settings)
	{
		this(settings, null);
	}
	
	/**
		Create a new filter with an optional chained parent filted.
		@param settings The settings object to use when checking if a file is installed or not
		@param parent An optional parent filter that will also be checked
	*/
	public InstalledFilter(Settings settings, Filter<ExtensionsFile> parent)
	{
		this.settings = settings;
		this.parent = parent;
	}
	
	/*
		From the Filter interface
		-------------------------
	*/
	@Override
	public boolean evaluate(ExtensionsFile xtFile)
	{
		if (parent != null && !parent.evaluate(xtFile)) 
		{
			return false;
		}
		File file = xtFile.getFile();
		if (file != null && !settings.isInstalledFile(file))
		{
			log.info("File is not installed (skipping): " + xtFile);
			return false;
		}
		return true;
	}
	// -------------------------
	
	
}
