/**
	$Id: WellCoordinateFormatter.java 4856 2009-03-27 10:14:41Z nicklas $

	Copyright (C) 2006 Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
*/
package net.sf.basedb.util.formatter;

import net.sf.basedb.util.Coordinate;


/**
	Formats an internal well coordinate to an external coordinate. There
	are two variants. One that converts to alphabetic values:
	0 -> A, 1 -> B, and so on. And one that converts between 0-based
	and 1-based coordinates.
	
	@author nicklas
	@version 2.12
	@base.modified $Date: 2009-03-27 11:14:41 +0100 (Fri, 27 Mar 2009) $
*/
public class WellCoordinateFormatter
	implements Formatter<Integer>
{
	private boolean toAlpha;
	/**
		Create a well coordinate formatter.
		@param toAlpha TRUE to convert to alphabetic coordinates, FALSE
			to convert from 0-based to 1-based coordinates
	*/
	public WellCoordinateFormatter(boolean toAlpha)
	{
		this.toAlpha = toAlpha;
	}
	
	/*
		From the Formatter interface
		-------------------------------------------
	*/
	@Override
	public String format(Integer value)
	{
		if (value == null) return null;
		int v = value+1;
		return toAlpha ? Coordinate.numericToAlpha(v) : Integer.toString(v);
	}
	public Integer parseString(String value)
	{
		if (value == null) return null;
		return Coordinate.alphaToNumeric(value);
	}
	// -------------------------------------------
	
}
