/**
	$Id $

	Copyright (C) 2011 Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
*/
package net.sf.basedb.util.overview.filter;

import net.sf.basedb.util.filter.Filter;
import net.sf.basedb.util.overview.Node;
import net.sf.basedb.util.overview.NodeAttribute;

/**
	A filter implementation that checks if a value for a given attribute
	existsin on a node. It will evaluate to true for all {@link Node}:s that
	has a value for a specific attribute.

	@author Nicklas
	@since 3.0
	@base.modified $Date $
*/
public class HasAttributeFilter
	implements Filter<Node>
{
	
	private final NodeAttribute<?> attribute;
	
	/**
		Create a filter that finds nodes that has value for the given
		attribute.
		@param attribute The attribute to check
	*/
	public HasAttributeFilter(NodeAttribute<?> attribute)
	{
		this.attribute = attribute;
	}

	/*
		From the Filter interface
		-------------------------
	*/
	@Override
	public boolean evaluate(Node node)
	{
		return node.hasAttribute(attribute);
	}
	// -----------------------------
}
