/*
	$Id: OneTimeFix.java 5735 2011-09-13 13:51:08Z nicklas $

	Copyright (C) 2005 Samuel Andersson, Jari Häkkinen, Nicklas Nordborg
	Copyright (C) 2006 Jari Häkkinen, Nicklas Nordborg, Martin Svensson

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
*/
package net.sf.basedb.install;

import java.sql.Connection;
import java.sql.DatabaseMetaData;
import java.sql.Driver;
import java.sql.DriverManager;
import java.sql.SQLException;
import java.util.Properties;


import net.sf.basedb.core.Application;
import net.sf.basedb.core.Config;
import net.sf.basedb.core.ProgressReporter;

/**
	This class is used for one-time fixes that are needed to be
	executed manually before or after an updgrade.

	@author Nicklas
	@version 2.7.2
	@base.modified $Date: 2011-09-13 15:51:08 +0200 (Tue, 13 Sep 2011) $
*/
public class OneTimeFix
{
	public static void main(String[] args)
	{
		
		if (args.length == 0) 
		{
			showUsage();
			return;
		}
		try
		{
			String cmd = args[0];		
			if ("foo".equals(cmd))
			{
				showUsage();
			}
			else
			{
				showUsage();
			}
		}
		catch (Throwable t)
		{
			System.out.println("\n");
			t.printStackTrace();
		}
	}

	private static void showUsage()
	{
		System.out.println("Usage: onetimefix.sh cmd <options>");
		System.out.println("generic options: ");
		System.out.println("    -u <login>    : Login name for the root user account; (root)");
		System.out.println("    -p <password> : Password for the root user account; (root)");
	}

	private static void showDbInfo(ProgressReporter progress)
	{
		final Properties properties = System.getProperties();
		final String jdbcDriver = Config.getString("db.driver");
		String driverVersion = "unknown version";
		try
		{
			final Driver driver = (Driver)Class.forName(jdbcDriver).newInstance();
			driverVersion = "version " + driver.getMajorVersion() + "." + driver.getMinorVersion();
		}
		catch (Throwable t)
		{}
		
		String message = 
			"--System information-----------------------------\n" + 
			"BASE     : " + Application.getMajorVersion() + "." +
				Application.getMinorVersion() + "." + 
				Application.getMaintenanceVersion() + "\n" + 
			"Database : " + Config.getString("db.url") + "\n" + 
			"Dialect  : " + Config.getString("db.dialect") + "\n" + 
			"JDBC     : " + jdbcDriver + "; " + driverVersion + "\n" +
			"Java     : " + properties.getProperty("java.runtime.name") + "; " +
				properties.getProperty("java.runtime.version") + "; " +
				properties.getProperty("java.vendor") + "\n" + 
			"OS       : " + properties.getProperty("os.name") +"; " +
				properties.getProperty("os.arch") + "; " +
				properties.getProperty("os.version") +"\n" +
			"-------------------------------------------------";
		if (progress != null)
		{
			progress.display(0, message);
		}
		else 
		{
			System.out.println(message);
		}
	}
	
	public static String getOption(String[] args, String option, String defaultValue)
	{
		String value = defaultValue;
		for (int i = 0; i < args.length; ++i)
		{
			if (option.equals(args[i]))
			{
				if (i+1 < args.length && !args[i+1].startsWith("-"))
				{
					value = args[i+1];
				}
				else
				{
					value = option;
				}
				return value;
			}
		}
		return value;
	}
	
	
	/**
		Get a JDBC {@link Connection} to the current BASE database.
		@since 2.9
	*/
	private static Connection getConnection()
		throws SQLException
	{
		String dbUrl = Config.getString("db.url");
		String dbUser = Config.getString("db.username");
		String dbPassword = Config.getString("db.password");
		return DriverManager.getConnection(dbUrl, dbUser, dbPassword);
	}
	
	/**
		Checks if the database is MySQL.
		@since 2.9
	*/
	private static boolean isMySQL()
		throws Exception
	{
		Connection c = null;
		try
		{
			c = getConnection();
			DatabaseMetaData metaData = c.getMetaData();
			String dbName = metaData.getDatabaseProductName();
			return dbName.contains("MySQL");
		}
		finally
		{
			if (c != null) c.close();
		}
	}
	

	public static void waitForEnter()
	{
		try
		{
			System.in.read();
			System.in.skip(System.in.available());
		}
		catch (Exception ex)
		{}
	}

}

