/**
	$Id: TestWebservices.java 5827 2011-10-26 10:51:52Z nicklas $

	Copyright (C) 2007 Martin Svensson, Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
*/

import net.sf.basedb.core.BaseException;
import net.sf.basedb.core.Config;
import net.sf.basedb.core.DataFileType;
import net.sf.basedb.core.Item;
import net.sf.basedb.core.Platform;
import net.sf.basedb.core.Type;
import net.sf.basedb.info.AnnotationInfo;
import net.sf.basedb.info.AnnotationTypeInfo;
import net.sf.basedb.info.ArrayDesignInfo;
import net.sf.basedb.info.BioAssaySetInfo;
import net.sf.basedb.info.DataFileTypeInfo;
import net.sf.basedb.info.ExperimentInfo;
import net.sf.basedb.info.ExtendedPropertyInfo;
import net.sf.basedb.info.FileInfo;
import net.sf.basedb.info.FileSetMemberInfo;
import net.sf.basedb.info.ProjectInfo;
import net.sf.basedb.info.QueryOptions;
import net.sf.basedb.info.QueryParameter;
import net.sf.basedb.info.RawBioAssayInfo;
import net.sf.basedb.info.ReporterListInfo;
import net.sf.basedb.util.FileUtil;
import net.sf.basedb.ws.client.AnnotationTypeClient;
import net.sf.basedb.ws.client.ArrayDesignClient;
import net.sf.basedb.ws.client.BioAssaySetClient;
import net.sf.basedb.ws.client.ExperimentClient;
import net.sf.basedb.ws.client.FileClient;
import net.sf.basedb.ws.client.ProjectClient;
import net.sf.basedb.ws.client.RawBioAssayClient;
import net.sf.basedb.ws.client.ReporterClient;
import net.sf.basedb.ws.client.ServiceFactory;
import net.sf.basedb.ws.client.SessionClient;

import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.util.Arrays;

/**
 	@author Martin, Nicklas
  	@since 2.5
 */
public class TestWebservices 
{
	static boolean ok = true;
	
	/**	
	  	@param args
	*/
	public static void main(String[] args)
	{
		TestUtil.checkArgs(args);
		TestUtil.begin();
		ok = test_all();
		TestUtil.stop();
	}
	
	static boolean test_all()
	{
		write("++Testing webservices");

		//Tests
		SessionClient client = test_SessionClient(null);
		if (client == null) return ok;
		
		boolean wasSetup = TestAnalyzeUtil.setupExperiment();
		int experimentId = TestAnalyzeUtil.getItemId("experiment.genepix");
		int rootBasId = TestAnalyzeUtil.getItemId("bioassayset.root");

		if (experimentId == 0 || rootBasId == 0)
		{
			ok = false;
			write("Test experiment was not setup correctly.");
			return ok;
		}

		//Create items to use in the tests		
		int projectId = TestProject.test_create("Web service project", true);

		int reporterListId1 = TestReporterList.test_create("net.sf.basedb.clients.test.reporterlist1", true);
		TestReporterList.test_add_reporters(reporterListId1, 50, 0);
		TestReporterList.test_setExperiment(reporterListId1, experimentId);
		
		// Export BASEfile:s
		int serialId = TestFile.test_create("serial.txt", false, false);
		int matrixId = TestFile.test_create("matrix.txt", false, false);
		TestBaseFileExporterImporter.test_export(rootBasId, serialId, false, false);
		TestBaseFileExporterImporter.test_export(rootBasId, matrixId, true, false);
		
		//Experiment with rawdata and array design stored in files.
		int cdfId = TestFile.test_create("data/test.affymetrix.cdf", false, false);
		int celId = TestFile.test_create("data/test.affymetrix.cel", false, false);
		int affyDesignId = TestArrayDesign.test_create(Platform.AFFYMETRIX, true);
		TestArrayDesign.test_set_file(affyDesignId, DataFileType.AFFYMETRIX_CDF, cdfId);
		int affyRaw1 = TestRawBioAssay.test_create(Platform.AFFYMETRIX, null, "Affymetrix.1", 0, 0, 0, 0, affyDesignId, false);
		TestDataFileType.test_set_file(Item.RAWBIOASSAY, affyRaw1, "affymetrix.cel", celId);
		int affyRaw2 = TestRawBioAssay.test_create(Platform.AFFYMETRIX, null, "Affymetrix.2", 0, 0, 0, 0, affyDesignId, false);
		TestDataFileType.test_set_file(Item.RAWBIOASSAY, affyRaw2, "affymetrix.cel", celId);
		
		int affyExperimentId = TestExperiment.test_create("platform.affymetrix", true);
		TestExperiment.test_add_rawbioassay(affyExperimentId, affyRaw1, affyRaw2);

		//Tests
		test_ProjectClient(client, projectId);
		test_AnnotationTypes(client);
		int genepixRaw1 = test_getRawDataInExperiment(client, experimentId, "Genepix.1");
		test_getRawDataInExperiment(client, affyExperimentId, "Affymetrix.1");	

		//Test to download a file which name in BASE starts with a number.
		TestFile.test_rename(celId, "2rawdataFile.txt");
		test_getRawDataInExperiment(client, affyExperimentId, "Affymetrix.2");
		
		// Get the array design that is linked with a raw bioassay
		test_getArrayDesign(client, genepixRaw1);
		test_getArrayDesign(client, affyRaw1);
		
		test_BioAssaySetClient(client, experimentId);
		test_getExperimentalFactorValues(client, experimentId);
		test_ReportersInExperiment(client, experimentId, reporterListId1);
		test_Logout(client);
		
		if (TestUtil.waitBeforeDelete()) TestUtil.waitForEnter();
		
		//Standard test:Delete/cleanup
		TestReporterList.test_delete(reporterListId1);
		
		TestExperiment.test_delete(affyExperimentId);
		TestProject.test_delete(projectId);
		
		TestRawBioAssay.test_delete(affyRaw1);
		TestRawBioAssay.test_delete(affyRaw2);
		
		TestArrayDesign.test_delete(affyDesignId);
		TestFile.test_delete(cdfId);
		TestFile.test_delete(celId);

		TestAnalyzeUtil.cleanupExperiment();
		
		write("++Testing webservices "+(ok ? "OK" : "Failed")+"\n");
		return ok;
	}

	static void write(String message)
	{
		System.out.println(message);
	}

	/**
	 	Test for the BioAssaySetClient.
	 	<ul>
	 		<li>Get bioassaysets in an experiment
	 		<li>Get available data files for the first bioassay set
	 		<li>Download each available data file
	 	</ul>
	  	@param client A logged in client.
	 */
	static void test_BioAssaySetClient(SessionClient client, int experimentId)
	{
		if (client == null || experimentId == 0) return;
		BioAssaySetClient basClient = new BioAssaySetClient(client);
		ExperimentClient experimentClient = new ExperimentClient(client);
		FileClient fileClient = new FileClient(client);
		
		int bioAssaySetId = 0;
		try
		{
			BioAssaySetInfo[] basInfo = experimentClient.getBioAssaySets(experimentId, new QueryOptions());
			if (basInfo == null || basInfo.length == 0) 
			{
				throw new BaseException("Could not load bioassays of experiment: id=" + experimentId);
			}
			bioAssaySetId = basInfo[0].getId();
			write ("--Get BioAssaySets from Experiment OK");
		}
		catch (Throwable t)
		{
			write ("--Get BioAssaySets from Experiment FAILED");
			t.printStackTrace();
			ok = false;
			return;
		}
		
		try
		{
			FileSetMemberInfo[] members = basClient.getDataFiles(bioAssaySetId, null); 
			if (members == null || members.length == 0) 
			{
				throw new Exception("Cant find any data files on bioassay set with id = " + bioAssaySetId);
			}
			
			for (FileSetMemberInfo info : members)
			{
				DataFileTypeInfo fileType = info.getDataFileTypeInfo();
				FileInfo file = info.getFileInfo();
				long expectedSize = file.getSize();
				File receivedFile = new File(file.getName());
				OutputStream out = new FileOutputStream(receivedFile);
				write("--Downloading file from BioAssaySet: " + fileType.getExternalId());
				InputStream in = fileClient.download(file.getId());
				if (in == null)
				{
					throw new BaseException("Download failed: InputStream==null");
				}
				FileUtil.copy(in, out);
				in.close();
				out.close();
				if (receivedFile.length() != expectedSize)
				{
					throw new IOException("Download failed. Got " + 
							receivedFile.length() + " bytes, expected " + expectedSize);
				}
				write("--Download OK; size=" + receivedFile.length() + " bytes");
				receivedFile.delete();
			}
			write("--Test of BioAssaySetClient OK");
		}
		catch (Throwable t)
		{
			write ("--Test of BioAssaySetClient FAILED");
			t.printStackTrace();
			ok = false;
		}
	}
	
	/**
	 	Following items are tested in raw bioassay service and array design service.
	 	<ul>
	 		<li>Get info about an array design used in a rawBioAssayId
	 		<li>Check if an array design has downloadable data.
	 		<li>Get the data file types used in an array design
	 		<li>Download the first of the data file-types.
	 	</ul>
	  	@param client A logged in client.
	  	@param rawBioAssayId Id of the RawBioAssay to test with.
	 */
	static void test_getArrayDesign(SessionClient client, int rawBioAssayId)
	{
		if (client == null || rawBioAssayId == 0) return;
		ArrayDesignClient arrayDesignClient = new ArrayDesignClient(client);
		RawBioAssayClient rawClient = new RawBioAssayClient(client);	
		FileClient fileClient = new FileClient(client);
		ArrayDesignInfo designInfo = null;
		
		try
		{					
			designInfo = rawClient.getArrayDesign(rawBioAssayId);			
			write ("--Get info about ArrayDesign OK");
		}
		catch (Throwable t)
		{
			write ("--Get info about ArrayDesign FAILED");
			t.printStackTrace();
			ok = false;
			return;
		}

		try
		{
			int arrayDesignId = designInfo.getId();
			boolean hasDownloadableData = arrayDesignClient.hasDownloadableData(arrayDesignId);
			FileSetMemberInfo[] members = arrayDesignClient.getDataFiles(arrayDesignId, new QueryOptions());
				
			if (hasDownloadableData && members.length > 0)
			{
				DataFileTypeInfo fileType = members[0].getDataFileTypeInfo();
				FileInfo file = members[0].getFileInfo();
				long expectedSize = file.getSize();
				write("--Downloading " + fileType.getName() + " for array design '" + designInfo.getName() + "': " + file.getName());
				File receivedFile = new File(file.getName());
				OutputStream out = new FileOutputStream(receivedFile);
				InputStream in = fileClient.download(file.getId());
				if (in == null)
				{
					throw new NullPointerException("Download failed: InputStream==null.");
				}
				FileUtil.copy(in, out);
				in.close();
				out.close();
				if (receivedFile.length() != expectedSize)
				{
					throw new IOException("Download failed. Got " + 
							receivedFile.length() + " bytes, expected " + expectedSize);
				}
				write("--Download OK; size=" + receivedFile.length() + " bytes");
				receivedFile.delete();
			}
			write("--Test of ArrayDesignClient OK");
		}
		catch (Throwable t)
		{
			write ("--Test of ArrayDesignClient FAILED");
			t.printStackTrace();
			ok = false;
		}
	}
	
	/**
	 	This tests following in experiment service and raw bioassay service:
	 	<ul>
	 		<li>Get experiment with a specific id
	 		<li>Get rawbioassay with a specific id from an experiment.
	 		<li>Checks if the rawbioassay has downloadable data.
	 		<li>Get the filetypes used in the raw bioassay.
	 		<li>Download raw data for the first filetype.
	 	</ul>
	  	@param client A logged in client.
	  	@param experimentId Id of the Experiment to test with. 
	  	@param rawBioAssayName Name of the RawBioAssay to test with.
	 */
	static int test_getRawDataInExperiment(SessionClient client, int experimentId, String rawBioAssayName)
	{
		if (client == null || experimentId == 0) return 0;
		int rawBioAssayId = 0;
		
		ExperimentClient experimentClient = new ExperimentClient(client);
		RawBioAssayClient rawClient = new RawBioAssayClient(client);
		FileClient fileClient = new FileClient(client);
		try
		{
			ExperimentInfo experimentInfo = experimentClient.getById(experimentId);
			write("--Get experiment with id " + experimentId + " OK");
		}
		catch (Throwable t)
		{
			write ("--Get experiment with id " + experimentId + " FAILED");
			t.printStackTrace();
			ok = false;
			return 0;
		}
		
		try
		{
			QueryOptions qOpt = new QueryOptions();
			qOpt.setRestrictionWithPrefix("$name=:name", "$");
			qOpt.setParameters(new QueryParameter[]{ new QueryParameter("name", rawBioAssayName)});
			RawBioAssayInfo[] rawInfo = experimentClient.getRawBioAssays(experimentId, qOpt);
			if (rawInfo.length < 1) 
			{
				throw new BaseException("Failed to load rawbioassay '" + rawBioAssayName + "'");
			}
			RawBioAssayInfo raw = rawInfo[0];
			rawBioAssayId = raw.getId();
			
			Boolean hasDownloadableData = rawClient.hasDownloadableData(rawBioAssayId);
			FileSetMemberInfo[] members = rawClient.getDataFiles(rawBioAssayId, null);
			
			if (hasDownloadableData && members.length > 0)
			{
				DataFileTypeInfo fileType = members[0].getDataFileTypeInfo();
				FileInfo file = members[0].getFileInfo();
				long expectedSize = file.getSize();
				write("--Downloading " + fileType.getName() + " for RawBioAssay '" + raw.getName() + "': " + file.getName());
				File receivedFile = new File(file.getName());
				OutputStream out = new FileOutputStream(receivedFile);
				InputStream in = fileClient.download(file.getId());
				if (in == null)
				{
					throw new NullPointerException("Download failed: InputStream==null.");
				}
				else
				{
					FileUtil.copy(in, out);
					in.close();
					out.close();
					if (receivedFile.length() != expectedSize)
					{
						throw new IOException("Download failed. Got " + 
								receivedFile.length() + " bytes, expected " + expectedSize);
					}
					write("--Download OK; size=" + receivedFile.length() + " bytes");
					receivedFile.delete();
				}
			}
			write ("--Test of RawBioAssayClient OK");
		}
		catch(Throwable t)
		{
			write ("--Test of RawBioAssayClient FAILED");
			t.printStackTrace();
			ok = false;
		}
		return rawBioAssayId;
	}
	
	
	/**
	 	This tests getting the experimental factors for an experiment and
	 	listing values for each raw bioassay.
	  	@param client A logged in client.
	  	@param experimentId Id of the Experiment to test with. 
	*/
	static void test_getExperimentalFactorValues(SessionClient client, int experimentId)
	{
		if (client == null || experimentId == 0) return;
		
		ExperimentClient experimentClient = new ExperimentClient(client);
		RawBioAssayClient rawClient = new RawBioAssayClient(client);
		QueryOptions qOpt = new QueryOptions();

		try
		{
			AnnotationTypeInfo[] factors = experimentClient.getExperimentalFactors(experimentId, qOpt);
			RawBioAssayInfo[] rawInfo = experimentClient.getRawBioAssays(experimentId, qOpt);

			StringBuilder header = new StringBuilder("\t");
			Integer[] annotationTypes = new Integer[factors.length];
			for (int i = 0; i < factors.length; ++i)
			{
				annotationTypes[i] = factors[i].getId();
				header.append(factors[i].getName()).append("\t");
			}
			
			write(header.toString());
			for (RawBioAssayInfo rba : rawInfo)
			{
				StringBuilder row = new StringBuilder();
				row.append(rba.getName()).append("\t");
				AnnotationInfo[] annotations = rawClient.getAnnotationValues(rba.getId(), 
					annotationTypes, true);
				
				for (AnnotationInfo info : annotations)
				{
					row.append(Arrays.asList(info.getValues()).toString()).append("\t");
				}
				write(row.toString());
			}
			
			write ("--Test of getExperimentalFactorValues OK");
		}
		catch(Throwable t)
		{
			write ("--Test of getExperimentalFactorValues FAILED");
			t.printStackTrace();
			ok = false;
		}
		
	}	
	
	static void test_ProjectClient(SessionClient client, int projectId)
	{
		if (client == null || projectId == 0) return;
		
		ProjectClient projectClient = new ProjectClient(client);
		try
		{
			//Tests ProjectClient.getProjects(QueryOptions qOpt)
			QueryOptions qOpt = new QueryOptions();
			qOpt.setRestrictionWithPrefix("$id="+projectId, "$");
			ProjectInfo[] infos = projectClient.getProjects(qOpt);
			if (!(infos != null && infos.length > 0))
			{				
				throw new Exception("No project was received");
			}
			
			//Tests ProjectClient.setActive(int projectId)
			projectClient.setActive(projectId);			
			write("--Test of ProjectClient OK");
		}		
		catch (Throwable t)
		{
			write ("--Test of ProjectClient FAILED");
			t.printStackTrace();
			ok = false;
		}		
	}
	
	static void test_AnnotationTypes(SessionClient client)
	{
		if (client == null) return;
		
		AnnotationTypeClient atClient = new AnnotationTypeClient(client);
		try
		{
			//Tests ProjectClient.getProjects(QueryOptions qOpt)
			QueryOptions qOpt = new QueryOptions();
			qOpt.setRestrictionWithPrefix("$valueType = :vt", "$");
			qOpt.setParameters(new QueryParameter[]{ new QueryParameter("vt", Type.STRING.getValue()) });
			AnnotationTypeInfo[] infos = atClient.getAnnotationTypes(null, qOpt);
			
			for (AnnotationTypeInfo info : infos)
			{
				write(info.getId() +"\t" + info.getName() + "\t" + info.getValueType() + "\t" + 
					Arrays.asList(info.getItemTypes()));
			}
			
			write("--Test of AnnotationTypeClient OK");
		}
		catch (Throwable t)
		{
			write ("--Test of AnnotationTypeClient FAILED");
			t.printStackTrace();
			ok = false;
		}		
	}	
	
	static SessionClient test_SessionClient(ServiceFactory serviceFactory)
	{
		SessionClient client = null;
		String url = Config.getString("test.webservices.url");
		if (url == null)
		{
			write("--Skipping webservices test: setting base.config[test.webservices.url] not found");
			return null;
		}
		try
		{
			write("--Using url: " + url);
			client = new SessionClient(url, null, TestUtil.getClient());
			client.login(TestUtil.getLogin(), TestUtil.getPassword(), "Test webservices");
	        write("--Create/login session OK: ID=" + client.getId());
		}
		catch (Throwable t)
		{
			write("--Create/login session FAILED");
			t.printStackTrace();
			return null;
		}
		try
		{
			client.refreshSession(client.getId());
			write("--Refresh session OK \tSession timeout set to: " + client.getSessionTimeOut() + " minutes");
		}
		catch(Throwable t)
		{
			write ("--Refresh session or get session's timeout FAILED");
			t.printStackTrace();
			ok = false;
		}		
		return client;
	}
	
	static void test_Logout(SessionClient client)
	{
		if (client == null) return;
		try
		{
			client.logout();
	      write("--Logout OK");
		}
		catch (Throwable t)
		{
			write("--Logout FAILED");
			t.printStackTrace();
			ok = false;
		}
	}
	
	static void test_ReportersInExperiment(SessionClient client, int experimentId, int reporterListId)
	{
		if (client == null || experimentId == 0 || reporterListId == 0)return;
		ExperimentClient experimentClient = new ExperimentClient(client);
		ReporterClient reporterClient = new ReporterClient(client);
		try
		{
			ReporterListInfo[] listInfo = experimentClient.getReporterLists(experimentId, null);
			write(listInfo.length > 0 ? 
					"--Received reporterlists(" + listInfo.length + ")" : 
					"--No reporterlists were received");
			
			ExtendedPropertyInfo[] extPropertyInfos = reporterClient.getExtendedProperties();
			write(extPropertyInfos.length > 0 ?
					"--Received extended properties for reporters(" + extPropertyInfos.length + ")" : 
					"--No extended properties were received");
			
			if (listInfo.length > 0 && extPropertyInfos.length > 0)
			{				
				write("--Downloading reporterlist '" + listInfo[0].getName() +"'...." );
				File receivedFile = new File(listInfo[0].getName() + ".txt");
				OutputStream out = new FileOutputStream(receivedFile);
				 
				InputStream in = reporterClient.downloadReportersInList(listInfo[0].getId(), extPropertyInfos, new QueryOptions());
				if (in == null)
				{
					throw new BaseException ("Download failed: InputStream==null");
				}
				FileUtil.copy(in, out);
				in.close();
				out.close();
				write("--Download OK; size=" + receivedFile.length() + " bytes");
				receivedFile.delete();
				
				receivedFile = null;
				out = null;
				
				//Test to receive reporters from service, at most 50 reporters				
				write("--Downloading reporters....");				
				receivedFile = new File("ReporterFile.txt");
				out = new FileOutputStream(receivedFile);
				in = null;
				QueryOptions qOpt = new QueryOptions();
				in = reporterClient.downloadReporters(extPropertyInfos, qOpt);
				
				if (in == null)
				{
					throw new BaseException("Download failed: InputStream==null");
				}
				FileUtil.copy(in, out);
				in.close();
				out.close();
				write("--Download OK; size=" + receivedFile.length() + " bytes");
				receivedFile.delete();
			}
				
			write("--Test of ReporterService OK");
		}
		catch (Throwable t)
		{
			write ("--Test of getReporterLists FAILED");
			t.printStackTrace();
			ok = false;
		}
	}
}
